// //////////////////////////////////////
// //////////////////////////////////////
// function serial_port receive        //
// //////////////////////////////////////
// This process receives data from the //
// serial port and save it in a file   //
// It is run as a thread               //
// //////////////////////////////////////


#include "serialframe.h"

void * funct_serialreceive (void * globaldatain) {
globaldatastr * globaldata;

// pointer to allpacketsend -> data returned from
// thread to main program
int * p_allpacketsend;


int stop;
int dummysend;
int donglefd;

// fileformat
// can be 1 for .ambe files (text)
// or 2 for .dvtool files (binary)
int fileformat;

FILE *  fileout;
int cs,sec;
int ret;

// total number of frames, is at least 1 (the start frame)
int dvtool_framecount=1;

unsigned char serialinbuff[8192];

// data-structures for .dvtool files

char dvtool_header[] = {0x44,0x56,0x54,0x4f,0x4f,0x4c,0x00,0x00,0x00,0x00,0x38,0x00,0x44,0x53,0x56,0x54,0x10,0x00,0x81,0x00,0x20,0x00,0x01,0x02,0xe8,0x4f,0x80,0x00,0x00,0x00,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0xff,0xff};
int dvtool_headersize=68;

char dvtool_line[] = {0x1b,0x00,0x44,0x53,0x56,0x54,0x20,0x00,0x81,0x00,0x20,0x00,0x01,0x02,0xe8,0x4f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x55,0x2d,0x16};
int dvtool_linesize=29;


// import data from global data
globaldata = (globaldatastr *) globaldatain;

donglefd=globaldata->donglefd;
p_allpacketsend=globaldata->p_allpacketsend;

fileformat=globaldata->fileformat;

if ((fileformat != 1) && (fileformat != 2)) {
	fprintf(stderr,"Error: invalid file-format parameter in serialreceive: %d\n",fileformat);
	// mark thread as finished
*globaldata->p_welive_serialreceive = 0;
	exit;
}; // end if

stop=0;


if (fname_out == NULL) {
	fprintf(stderr,"Error: output filename unknown!\n");
} else {
	if (fileformat == 1) {
	// ambe files are text-files
		fileout=fopen(fname_out,"w");
	} else {
	// dvtool files are binary files
		fileout=fopen(fname_out,"wb");
	}; // end else - if
}; // end else - if

if (fileout == 0) {
        fprintf(stderr,"Unable to open output file! %s\n",fname_out);
        exit(-1);
}; // end if


// write header in output file

if (fileformat == 1) {
	// AMBE file-format

	// Info: lines beginning with "#" are comment and can be ignore.
	// however, for future expansion, the follow lines have be reserved:
	// #C Version: version of format: currently "1.0"
	// #C Name: name of file
	// #C Info: any additional info that can be usefull
	ret=fprintf(fileout,"#C Version 1.0\n");


} else {
	// DVTOOL file-format
	// Info: .dvtool files begin with a fixed 10 byte header (containing
	// "DVTOOL" + 4 bytes lenth) + 1 start-frame of 58 bytes
	// The length-indication (bytes 6 up to 9) are now left blank
	// and will be filled up at the end of the program
	
	// write header to outfile
	ret=fwrite(dvtool_header,dvtool_headersize,1,fileout);
}; // end while

if (ret == 0) {
	fprintf(stderr,"Error writing to output file: %s!\n","/var/tmp/test.ambe");
	exit(-1);
}; // end if


while (stop == 0) {
	uint16_t plen_i;
	uint16_t ptype;

	int ret;


	// get next frame, correction is enabled, 2 ms of delay if no data
	// and 1 second timeout (500 times 2 ms)
	ret=serialframereceive(donglefd,serialinbuff,&ptype,&plen_i,1,2000,500);


	// we are only interested in ambe data (ptype = 5, length = 48 bytes)
	if ((ptype == 5) && (plen_i == 48)) {
		// write data in output file

		if (fileformat == 1) {
		// .ambe file
			fprintf(fileout,"%05d %02d %02x%02x%02x%02x%02x%02x%02x%02x%02x\n",sec,cs,serialinbuff[24],serialinbuff[25],serialinbuff[26],serialinbuff[27],serialinbuff[28],serialinbuff[29],serialinbuff[30],serialinbuff[31],serialinbuff[32]);

			cs += 2;

			if (cs >= 100) {
				sec++;
				cs=0;
			}; // end if
		} else {
			dvtool_framecount++;

			// copy 9 octets ambe-data: octets 24 up to 32 of the ambe-frame 
			// go into octets 17 to 25 of the dvtool line-frame
			// note: all octets are counted started from 0
			memcpy((void *) &dvtool_line[17],&serialinbuff[24],9);

			// write line to file
			if (fileout != NULL) {
				fwrite(dvtool_line,dvtool_linesize,1,fileout);
			}; // end if

			// Octet 16 of the dvtool line-frame contains a sequence-number
			// wrap around at 0x14 (=20)
			dvtool_line[16]++;

			if (dvtool_line[16] > 0x14) {
				dvtool_line[16]=0x00;
			}; // end if

		}; // end else - if (file format)
		
	}; // end if (ambe packet received)


	// if the last ambe packet was received, we need to do some
	// additional stuff for .dvtool files
	if ((allpacketsend >= 1) && (ptype == 5) && (fileout != NULL) &&
			(fileformat == 2)) {
		uint32_t framecount_smallendian;

if (DEBUG == 1) {
	fprintf(stderr,"DVTOOL last frame\n");
}; // end if

		// for the .dvtool file-format, we need to do two more things:
		// - send a termination-frame with all ambe-data to 0
		// and a sequence-number at normal + 0x40
		// - addapt the number-of-records counter in the beginning of the file

		// Termination-frame: 
		// clean all ambe data
		memset(&dvtool_line[17],0,9);

		// set "end-of-file" marker in sequence-number
		dvtool_line[16]+=0x40;

		// write line
		fwrite(dvtool_line,dvtool_linesize,1,fileout);

	
		// number-of-records counter
		fseek(fileout,6,SEEK_SET);

		framecount_smallendian=htonl(dvtool_framecount);
		fwrite((void *) &framecount_smallendian,4,1,fileout);

		// close file
		fclose(fileout);
		fileout=NULL;

	}; // end if ( last frame + addapt .dvtool files)

	// was this the last packet and was the stop send?
	if (allpacketsend == 2) {

		// yes, set marker to 4 and stop
		allpacketsend=3;
		stop=1;

	}; // end if (lastpacket)

}; // end while (stop)

// same additional things to do for .dvtool files


// close fileout
if (fileout != NULL) {
	fclose (fileout);
}; // end if

// mark thread as done
*globaldata->p_welive_serialreceive = 0;
}; // end if



